library ieee;
use ieee.std_logic_1164.all;
    
entity interface_mesh is
	port(							   
	ungated_clk			: in  std_logic;
	clk					: in  std_logic;
	reset					: in  std_logic;
	start_testing		: in std_logic;
	grant_l_33			: in  std_logic;
	x_in_l_33			: in  std_logic_vector(7 downto 0);
	request_l_33		: in  std_logic;
	mux_select			: in  std_logic_vector(3 downto 0);
	readout				: in  std_logic;
	readout_finished	: in  std_logic;				   
	BRAM_enable			: in  std_logic;
	BRAM_addr			: in  std_logic_vector(8 downto 0);
	x_out_l_33			: out std_logic_vector(7 downto 0);
	RTS_l_33			: out std_logic;
	CTS_l_33			: out std_logic;
	mux_o_packetcnt		: out std_logic_vector(31 downto 0);
	mux_o_tstamp		: out std_logic_vector(31 downto 0)
	);
end entity;

architecture behavioral of interface_mesh is

	signal grant_l_00,grant_l_01,grant_l_02,grant_l_03          				: std_logic;        
	signal grant_l_10,grant_l_11,grant_l_12,grant_l_13          				: std_logic;       
	signal grant_l_20,grant_l_21,grant_l_22,grant_l_23          				: std_logic;             
	signal grant_l_30,grant_l_31,grant_l_32                     				: std_logic;
	
	signal x_in_l_00,x_in_l_01,x_in_l_02,x_in_l_03              				: std_logic_vector(7 downto 0);
	signal x_in_l_10,x_in_l_11,x_in_l_12,x_in_l_13              				: std_logic_vector(7 downto 0);
	signal x_in_l_20,x_in_l_21,x_in_l_22,x_in_l_23              				: std_logic_vector(7 downto 0);
	signal x_in_l_30,x_in_l_31,x_in_l_32                        				: std_logic_vector(7 downto 0); 
	
	signal request_l_00,request_l_01,request_l_02,request_l_03      			: std_logic;
	signal request_l_10,request_l_11,request_l_12,request_l_13      			: std_logic;
	signal request_l_20,request_l_21,request_l_22,request_l_23      			: std_logic;
	signal request_l_30,request_l_31,request_l_32                   			: std_logic; 
	
	signal x_out_l_00,x_out_l_01,x_out_l_02,x_out_l_03                          : std_logic_vector(7 downto 0);
	signal x_out_l_10,x_out_l_11,x_out_l_12,x_out_l_13                          : std_logic_vector(7 downto 0);
	signal x_out_l_20,x_out_l_21,x_out_l_22,x_out_l_23                          : std_logic_vector(7 downto 0);
	signal x_out_l_30,x_out_l_31,x_out_l_32                                     : std_logic_vector(7 downto 0); 
	
	signal RTS_l_00,RTS_l_01,RTS_l_02,RTS_l_03                            	    : std_logic;
	signal RTS_l_10,RTS_l_11,RTS_l_12,RTS_l_13                            	    : std_logic;
	signal RTS_l_20,RTS_l_21,RTS_l_22,RTS_l_23                             		: std_logic;
	signal RTS_l_30,RTS_l_31,RTS_l_32                          				    : std_logic;
	
	signal CTS_l_00,CTS_l_01,CTS_l_02,CTS_l_03                                  : std_logic;
	signal CTS_l_10,CTS_l_11,CTS_l_12,CTS_l_13          						: std_logic;
	signal CTS_l_20,CTS_l_21,CTS_l_22,CTS_l_23                                  : std_logic;
	signal CTS_l_30,CTS_l_31,CTS_l_32                                           : std_logic;
	
	signal PacketTrigger_00,PacketTrigger_01,PacketTrigger_02,PacketTrigger_03	: std_logic;
	signal PacketTrigger_10,PacketTrigger_11,PacketTrigger_12,PacketTrigger_13	: std_logic;
	signal PacketTrigger_20,PacketTrigger_21,PacketTrigger_22,PacketTrigger_23	: std_logic;
	signal PacketTrigger_30,PacketTrigger_31,PacketTrigger_32,PacketTrigger_33	: std_logic;
	
	signal PacketCnt_00,PacketCnt_01,PacketCnt_02,PacketCnt_03					: integer;
	signal PacketCnt_10,PacketCnt_11,PacketCnt_12,PacketCnt_13					: integer;
	signal PacketCnt_20,PacketCnt_21,PacketCnt_22,PacketCnt_23					: integer;
	signal PacketCnt_30,PacketCnt_31,PacketCnt_32,PacketCnt_33					: integer;
	
	signal tstamp_out_00,tstamp_out_01,tstamp_out_02,tstamp_out_03				: std_logic_vector(31 downto 0);
	signal tstamp_out_10,tstamp_out_11,tstamp_out_12,tstamp_out_13				: std_logic_vector(31 downto 0);
	signal tstamp_out_20,tstamp_out_21,tstamp_out_22,tstamp_out_23				: std_logic_vector(31 downto 0);
	signal tstamp_out_30,tstamp_out_31,tstamp_out_32							: std_logic_vector(31 downto 0);
	
	signal global_time	: std_logic_vector(31 downto 0);
	
begin					   

---------------
--Multiplexer--
---------------

mux_packetcnt: entity noc_v1_00_a.mux(behavioral)
	port map(
	in_00	=> PacketCnt_00,
	in_01	=> PacketCnt_01,
	in_02	=> PacketCnt_02,
	in_03	=> PacketCnt_03,
	in_10	=> PacketCnt_10,
	in_11	=> PacketCnt_11,
	in_12	=> PacketCnt_12,
	in_13	=> PacketCnt_13,
	in_20	=> PacketCnt_20,
	in_21	=> PacketCnt_21,
	in_22	=> PacketCnt_22,
	in_23	=> PacketCnt_23,
	in_30	=> PacketCnt_30,
	in_31	=> PacketCnt_31,
	in_32	=> PacketCnt_32,
	in_33	=> PacketCnt_33,
	sel		=> mux_select,
	x_out	=> mux_o_packetcnt
	);

mux_tstamps: entity noc_v1_00_a.mux_tstamp(behavioral)
	port map(
	in_00	=> tstamp_out_00,
	in_01	=> tstamp_out_01,
	in_02	=> tstamp_out_02,
	in_03	=> tstamp_out_03,
	in_10	=> tstamp_out_10,
	in_11	=> tstamp_out_11,
	in_12	=> tstamp_out_12,
	in_13	=> tstamp_out_13,
	in_20	=> tstamp_out_20,
	in_21	=> tstamp_out_21,
	in_22	=> tstamp_out_22,
	in_23	=> tstamp_out_23,
	in_30	=> tstamp_out_30,
	in_31	=> tstamp_out_31,
	in_32	=> tstamp_out_32,
	sel		=> mux_select,
	x_out	=> mux_o_tstamp
	);
------------------
--GLOBAL COUNTER--
------------------

global_counter: entity noc_v1_00_a.global_counter(behavioral)
	port map(
	clk			=> clk,
	reset		=> reset,
	start_testing => start_testing,
	global_time	=> global_time
	);
-----------------
--4x4 RUTERMESH--
-----------------

mesh: entity noc_v1_00_a.noc_44_mesh(behavioral)
	port map(
	clk 			=> clk,
	reset			=> reset,
	grant_l_00		=> grant_l_00,
	grant_l_01		=> grant_l_01,
	grant_l_02		=> grant_l_02,
	grant_l_03		=> grant_l_03,
	grant_l_10		=> grant_l_10,
	grant_l_11		=> grant_l_11,
	grant_l_12		=> grant_l_12,
	grant_l_13		=> grant_l_13,
	grant_l_20		=> grant_l_20,
	grant_l_21		=> grant_l_21,
	grant_l_22		=> grant_l_22,
	grant_l_23		=> grant_l_23,
	grant_l_30		=> grant_l_30,
	grant_l_31		=> grant_l_31,
	grant_l_32		=> grant_l_32,
	grant_l_33		=> grant_l_33,
	x_in_l_00		=> x_in_l_00,
	x_in_l_01		=> x_in_l_01,
	x_in_l_02		=> x_in_l_02,
	x_in_l_03		=> x_in_l_03,
	x_in_l_10		=> x_in_l_10,
	x_in_l_11		=> x_in_l_11,
	x_in_l_12		=> x_in_l_12,
	x_in_l_13		=> x_in_l_13,
	x_in_l_20		=> x_in_l_20,
	x_in_l_21		=> x_in_l_21,
	x_in_l_22		=> x_in_l_22,
	x_in_l_23		=> x_in_l_23,
	x_in_l_30		=> x_in_l_30,
	x_in_l_31		=> x_in_l_31,
	x_in_l_32		=> x_in_l_32,
	x_in_l_33		=> x_in_l_33,
	request_l_00	=> request_l_00,
	request_l_01	=> request_l_01,
	request_l_02	=> request_l_02,
	request_l_03	=> request_l_03,
	request_l_10	=> request_l_10,
	request_l_11	=> request_l_11,
	request_l_12	=> request_l_12,
	request_l_13	=> request_l_13,
	request_l_20	=> request_l_20,
	request_l_21	=> request_l_21,
	request_l_22	=> request_l_22,
	request_l_23	=> request_l_23,
	request_l_30	=> request_l_30,
	request_l_31	=> request_l_31,
	request_l_32	=> request_l_32,
	request_l_33	=> request_l_33,
	x_out_l_00		=> x_out_l_00,
	x_out_l_01		=> x_out_l_01,
	x_out_l_02		=> x_out_l_02,
	x_out_l_03		=> x_out_l_03,
	x_out_l_10		=> x_out_l_10,
	x_out_l_11		=> x_out_l_11,
	x_out_l_12		=> x_out_l_12,
	x_out_l_13		=> x_out_l_13,
	x_out_l_20		=> x_out_l_20,
	x_out_l_21		=> x_out_l_21,
	x_out_l_22		=> x_out_l_22,
	x_out_l_23		=> x_out_l_23,
	x_out_l_30		=> x_out_l_30,
	x_out_l_31		=> x_out_l_31,
	x_out_l_32		=> x_out_l_32,
	x_out_l_33		=> x_out_l_33,
	RTS_l_00		=> RTS_l_00,
	RTS_l_01		=> RTS_l_01,
	RTS_l_02		=> RTS_l_02,
	RTS_l_03		=> RTS_l_03,
	RTS_l_10		=> RTS_l_10,
	RTS_l_11		=> RTS_l_11,
	RTS_l_12		=> RTS_l_12,
	RTS_l_13		=> RTS_l_13,
	RTS_l_20		=> RTS_l_20,
	RTS_l_21		=> RTS_l_21,
	RTS_l_22		=> RTS_l_22,
	RTS_l_23		=> RTS_l_23,
	RTS_l_30		=> RTS_l_30,
	RTS_l_31		=> RTS_l_31,
	RTS_l_32		=> RTS_l_32,
	RTS_l_33		=> RTS_l_33,
	CTS_l_00		=> CTS_l_00,
	CTS_l_01		=> CTS_l_01,
	CTS_l_02		=> CTS_l_02,
	CTS_l_03		=> CTS_l_03,
	CTS_l_10		=> CTS_l_10,
	CTS_l_11		=> CTS_l_11,
	CTS_l_12		=> CTS_l_12,
	CTS_l_13		=> CTS_l_13,
	CTS_l_20		=> CTS_l_20,
	CTS_l_21		=> CTS_l_21,
	CTS_l_22		=> CTS_l_22,
	CTS_l_23		=> CTS_l_23,
	CTS_l_30		=> CTS_l_30,
	CTS_l_31		=> CTS_l_31,
	CTS_l_32		=> CTS_l_32,
	CTS_l_33		=> CTS_l_33,
	PacketTrigger_00=>PacketTrigger_00,
	PacketTrigger_01=>PacketTrigger_01,
	PacketTrigger_02=>PacketTrigger_02,
	PacketTrigger_03=>PacketTrigger_03,
	PacketTrigger_10=>PacketTrigger_10,
	PacketTrigger_11=>PacketTrigger_11,
	PacketTrigger_12=>PacketTrigger_12,
	PacketTrigger_13=>PacketTrigger_13,
	PacketTrigger_20=>PacketTrigger_20,
	PacketTrigger_21=>PacketTrigger_21,
	PacketTrigger_22=>PacketTrigger_22,
	PacketTrigger_23=>PacketTrigger_23,
	PacketTrigger_30=>PacketTrigger_30,
	PacketTrigger_31=>PacketTrigger_31,
	PacketTrigger_32=>PacketTrigger_32,
	PacketTrigger_33=>PacketTrigger_33
	);				  			  
	
----------------------
--Traffic Generators--
----------------------

TG_00: entity noc_v1_00_a.TG(behavioral)
	port map(		   
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset,
	start_testing => start_testing,	
	x_in		=> x_out_l_00,
	request		=> RTS_l_00,
	grant		=> CTS_l_00,
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_00,
	RTS			=> request_l_00,
	CTS			=> grant_l_00,
	tstamp_out	=> tstamp_out_00
	);
	
TG_01: entity noc_v1_00_a.TG(behavioral)
	port map(  
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_01,
	request		=> RTS_l_01,
	grant		=> CTS_l_01, 
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_01,
	RTS			=> request_l_01,
	CTS			=> grant_l_01,
	tstamp_out	=> tstamp_out_01
	);

TG_02: entity noc_v1_00_a.TG(behavioral)
	port map(	 
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_02,
	request		=> RTS_l_02,
	grant		=> CTS_l_02,
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_02,
	RTS			=> request_l_02,
	CTS			=> grant_l_02,
	tstamp_out	=> tstamp_out_02
	);

TG_03: entity noc_v1_00_a.TG(behavioral)
	port map(	
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset,
	start_testing => start_testing, 
	x_in		=> x_out_l_03,
	request		=> RTS_l_03,
	grant		=> CTS_l_03,
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_03,
	RTS			=> request_l_03,
	CTS			=> grant_l_03,
	tstamp_out	=> tstamp_out_03
	);

TG_10: entity noc_v1_00_a.TG(behavioral)
	port map(
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_10,
	request		=> RTS_l_10,
	grant		=> CTS_l_10,
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_10,
	RTS			=> request_l_10,
	CTS			=> grant_l_10,
	tstamp_out	=> tstamp_out_10
	);

TG_11: entity noc_v1_00_a.TG(behavioral)
	port map(	  
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_11,
	request		=> RTS_l_11,
	grant		=> CTS_l_11,
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_11,
	RTS			=> request_l_11,
	CTS			=> grant_l_11,
	tstamp_out	=> tstamp_out_11
	);
	
TG_12: entity noc_v1_00_a.TG(behavioral)
	port map( 
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_12,
	request		=> RTS_l_12,
	grant		=> CTS_l_12,
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_12,
	RTS			=> request_l_12,
	CTS			=> grant_l_12,
	tstamp_out	=> tstamp_out_12
	);
	
TG_13: entity noc_v1_00_a.TG(behavioral)
	port map(  
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_13,
	request		=> RTS_l_13,
	grant		=> CTS_l_13,
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_13,
	RTS			=> request_l_13,
	CTS			=> grant_l_13,
	tstamp_out	=> tstamp_out_13
	);
	
TG_20: entity noc_v1_00_a.TG(behavioral)
	port map(  
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_20,
	request		=> RTS_l_20,
	grant		=> CTS_l_20, 
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_20,
	RTS			=> request_l_20,
	CTS			=> grant_l_20,
	tstamp_out	=> tstamp_out_20
	);
	
TG_21: entity noc_v1_00_a.TG(behavioral)
	port map(
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_21,
	request		=> RTS_l_21,
	grant		=> CTS_l_21,
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_21,
	RTS			=> request_l_21,
	CTS			=> grant_l_21,
	tstamp_out	=> tstamp_out_21
	);
	
TG_22: entity noc_v1_00_a.TG(behavioral)
	port map(
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_22,
	request		=> RTS_l_22,
	grant		=> CTS_l_22,
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_22,
	RTS			=> request_l_22,
	CTS			=> grant_l_22,
	tstamp_out	=> tstamp_out_22
	);
	
TG_23: entity noc_v1_00_a.TG(behavioral)
	port map(
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset,
	start_testing => start_testing,
	x_in		=> x_out_l_23,
	request		=> RTS_l_23,
	grant		=> CTS_l_23,  
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_23,
	RTS			=> request_l_23,
	CTS			=> grant_l_23,
	tstamp_out	=> tstamp_out_23
	);
	
TG_30: entity noc_v1_00_a.TG(behavioral)
	port map(
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_30,
	request		=> RTS_l_30,
	grant		=> CTS_l_30,  
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_30,
	RTS			=> request_l_30,
	CTS			=> grant_l_30,
	tstamp_out	=> tstamp_out_30
	);
	
TG_31: entity noc_v1_00_a.TG(behavioral)
	port map(
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_31,
	request		=> RTS_l_31,
	grant		=> CTS_l_31,
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_31,
	RTS			=> request_l_31,
	CTS			=> grant_l_31,
	tstamp_out	=> tstamp_out_31
	);
	
TG_32: entity noc_v1_00_a.TG(behavioral)
	port map(
	ungated_clk	=> ungated_clk,
	clk			=> clk,
	reset		=> reset, 
	start_testing => start_testing,
	x_in		=> x_out_l_32,
	request		=> RTS_l_32,
	grant		=> CTS_l_32, 
	SW_enable	=> BRAM_enable,
	SW_addr		=> BRAM_addr,
	global_time	=> global_time,
	x_out		=> x_in_l_32,
	RTS			=> request_l_32,
	CTS			=> grant_l_32,
	tstamp_out	=> tstamp_out_32
	);
	
--------------------
--Traffic Monitors--	
--------------------

TM_00: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_00,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_00
	);
	
TM_01: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_01,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_01
	);
	
TM_02: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_02,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_02
	);
	
TM_03: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_03,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_03
	);
	
TM_10: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_10,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_10
	);
	
TM_11: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_11,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_11
	);
	
TM_12: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_12,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_12
	);
	
TM_13: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_13,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_13
	);	
	
TM_20: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_20,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_20
	);
	
TM_21: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_21,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_21
	);
	
TM_22: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_22,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_22
	);
	
TM_23: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_23,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_23
	);
	
TM_30: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_30,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_30
	);
	
TM_31: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_31,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_31
	);
	
TM_32: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_32,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_32
	);
	
TM_33: entity noc_v1_00_a.TM(behavioral)
	port map(
	clk					=> ungated_clk,
	reset				=> reset,
	CTS					=> PacketTrigger_33,
	readout_finished	=> readout_finished,
	packet_counter		=> PacketCnt_33
	);
		 
		 
end architecture;