----------------------------------
--4x4 Network on Chip Mesh		--
--Written by: Andreas Heps     --
----------------------------------
--Summary: Module which connects--
--16 routers together and 		--
--presents the local signals	--
--at top level					--
----------------------------------

library ieee;
use ieee.std_logic_1164.all;
    
entity noc_44_mesh is
port(   clk,reset                                                            :in  STD_LOGIC;		
        grant_l_00,grant_l_01,grant_l_02,grant_l_03          				 :in  STD_LOGIC; 
        grant_l_10,grant_l_11,grant_l_12,grant_l_13          				 :in  STD_LOGIC;
        grant_l_20,grant_l_21,grant_l_22,grant_l_23          				 :in  STD_LOGIC;
        grant_l_30,grant_l_31,grant_l_32,grant_l_33           				 :in  STD_LOGIC;
        x_in_l_00,x_in_l_01,x_in_l_02,x_in_l_03                              :in  STD_LOGIC_VECTOR(7 downto 0);
        x_in_l_10,x_in_l_11,x_in_l_12,x_in_l_13                              :in  STD_LOGIC_VECTOR(7 downto 0);
        x_in_l_20,x_in_l_21,x_in_l_22,x_in_l_23                              :in  STD_LOGIC_VECTOR(7 downto 0);
        x_in_l_30,x_in_l_31,x_in_l_32,x_in_l_33                              :in  STD_LOGIC_VECTOR(7 downto 0);                        
        request_l_00,request_l_01,request_l_02,request_l_03      			 :in  STD_LOGIC;
        request_l_10,request_l_11,request_l_12,request_l_13      			 :in  STD_LOGIC;
        request_l_20,request_l_21,request_l_22,request_l_23     			 :in  STD_LOGIC;
        request_l_30,request_l_31,request_l_32,request_l_33     			 :in  STD_LOGIC;       
        x_out_l_00,x_out_l_01,x_out_l_02,x_out_l_03                          :out STD_LOGIC_VECTOR(7 downto 0);
        x_out_l_10,x_out_l_11,x_out_l_12,x_out_l_13                          :out STD_LOGIC_VECTOR(7 downto 0);
        x_out_l_20,x_out_l_21,x_out_l_22,x_out_l_23                          :out STD_LOGIC_VECTOR(7 downto 0);
        x_out_l_30,x_out_l_31,x_out_l_32,x_out_l_33                          :out STD_LOGIC_VECTOR(7 downto 0);     
        RTS_l_00,RTS_l_01,RTS_l_02,RTS_l_03                                  :out STD_LOGIC;
        RTS_l_10,RTS_l_11,RTS_l_12,RTS_l_13                                  :out STD_LOGIC;
        RTS_l_20,RTS_l_21,RTS_l_22,RTS_l_23                                  :out STD_LOGIC;
        RTS_l_30,RTS_l_31,RTS_l_32,RTS_l_33                                  :out STD_LOGIC;                         
        CTS_l_00,CTS_l_01,CTS_l_02,CTS_l_03                                  :out STD_LOGIC;
        CTS_l_10,CTS_l_11,CTS_l_12,CTS_l_13                                  :out STD_LOGIC;
        CTS_l_20,CTS_l_21,CTS_l_22,CTS_l_23                                  :out STD_LOGIC;
        CTS_l_30,CTS_l_31,CTS_l_32,CTS_l_33                                  :out STD_LOGIC;
		PacketTrigger_00,PacketTrigger_01,PacketTrigger_02,PacketTrigger_03					:out std_logic;
		PacketTrigger_10,PacketTrigger_11,PacketTrigger_12,PacketTrigger_13					:out std_logic;
		PacketTrigger_20,PacketTrigger_21,PacketTrigger_22,PacketTrigger_23					:out std_logic;
		PacketTrigger_30,PacketTrigger_31,PacketTrigger_32,PacketTrigger_33					:out std_logic		
);
end entity;
    
    
      
architecture behavioral of noc_44_mesh is
-----------------
--Dummy signals--
-----------------
signal d00,d01,d02,d03,d04,d05,d06,d07,d08,d09,d10,
d11,d12,d13,d14,d15,d16,d17,d18,d19,d20,d21,d22,d23,
d24,d25,d26,d27,d28,d29,d30,d31,d32	: std_logic_vector(7 downto 0) := (others=>'0');

signal dummy_CTS_RTS	: std_logic_vector(63 downto 0);

----------------------
--Coordinate signals--
----------------------
signal coord00	: std_logic_vector(3 downto 0);
signal coord01	: std_logic_vector(3 downto 0);
signal coord02	: std_logic_vector(3 downto 0);
signal coord03	: std_logic_vector(3 downto 0);
signal coord10	: std_logic_vector(3 downto 0);
signal coord11	: std_logic_vector(3 downto 0);
signal coord12	: std_logic_vector(3 downto 0);
signal coord13	: std_logic_vector(3 downto 0);
signal coord20	: std_logic_vector(3 downto 0);
signal coord21	: std_logic_vector(3 downto 0);
signal coord22	: std_logic_vector(3 downto 0);
signal coord23	: std_logic_vector(3 downto 0);
signal coord30	: std_logic_vector(3 downto 0);
signal coord31	: std_logic_vector(3 downto 0);
signal coord32	: std_logic_vector(3 downto 0);
signal coord33	: std_logic_vector(3 downto 0);

-------------------------
--signals from ruter 00--
-------------------------
signal x_out_00s_to_x_in_10n	: std_logic_vector(7 downto 0);
signal x_out_00e_to_x_in_01w	: std_logic_vector(7 downto 0);

signal RTS_00s_to_request_10n	: std_logic;
signal RTS_00e_to_request_01w	: std_logic;

signal CTS_00s_to_grant_10n		: std_logic;
signal CTS_00e_to_grant_01w		: std_logic;

-------------------------
--signals from ruter 01--
-------------------------
signal x_out_01w_to_x_in_00e	: std_logic_vector(7 downto 0);
signal x_out_01s_to_x_in_11n	: std_logic_vector(7 downto 0);
signal x_out_01e_to_x_in_02w	: std_logic_vector(7 downto 0);

signal RTS_01w_to_request_00e	: std_logic;
signal RTS_01s_to_request_11n	: std_logic;
signal RTS_01e_to_request_02w	: std_logic;

signal CTS_01w_to_grant_00e		: std_logic;
signal CTS_01s_to_grant_11n		: std_logic;
signal CTS_01e_to_grant_02w		: std_logic;

-------------------------
--signals from ruter 02--
-------------------------
signal x_out_02w_to_x_in_01e	: std_logic_vector(7 downto 0);
signal x_out_02s_to_x_in_12n	: std_logic_vector(7 downto 0);
signal x_out_02e_to_x_in_03w	: std_logic_vector(7 downto 0);

signal RTS_02w_to_request_01e	: std_logic;
signal RTS_02s_to_request_12n	: std_logic;
signal RTS_02e_to_request_03w	: std_logic;

signal CTS_02w_to_grant_01e		: std_logic;
signal CTS_02s_to_grant_12n		: std_logic;
signal CTS_02e_to_grant_03w		: std_logic;

-------------------------
--signals from ruter 03--
-------------------------
signal x_out_03w_to_x_in_02e	: std_logic_vector(7 downto 0);
signal x_out_03s_to_x_in_13n	: std_logic_vector(7 downto 0);

signal RTS_03w_to_request_02e	: std_logic;
signal RTS_03s_to_request_13n	: std_logic;

signal CTS_03w_to_grant_02e		: std_logic;
signal CTS_03s_to_grant_13n		: std_logic;

-------------------------
--signals from ruter 10--
-------------------------
signal x_out_10n_to_x_in_00s	: std_logic_vector(7 downto 0);
signal x_out_10s_to_x_in_20n	: std_logic_vector(7 downto 0);
signal x_out_10e_to_x_in_11w	: std_logic_vector(7 downto 0);

signal RTS_10n_to_request_00s	: std_logic;
signal RTS_10s_to_request_20n	: std_logic;
signal RTS_10e_to_request_11w	: std_logic;

signal CTS_10n_to_grant_00s		: std_logic;
signal CTS_10s_to_grant_20n		: std_logic;
signal CTS_10e_to_grant_11w		: std_logic;

-------------------------
--signals from ruter 11--
------------------------- 
signal x_out_11n_to_x_in_01s	: std_logic_vector(7 downto 0);
signal x_out_11w_to_x_in_10e	: std_logic_vector(7 downto 0);
signal x_out_11s_to_x_in_21n	: std_logic_vector(7 downto 0);
signal x_out_11e_to_x_in_12w	: std_logic_vector(7 downto 0);

signal RTS_11n_to_request_01s	: std_logic;
signal RTS_11w_to_request_10e	: std_logic;
signal RTS_11s_to_request_21n	: std_logic;
signal RTS_11e_to_request_12w	: std_logic;

signal CTS_11n_to_grant_01s		: std_logic;
signal CTS_11w_to_grant_10e		: std_logic;
signal CTS_11s_to_grant_21n		: std_logic;
signal CTS_11e_to_grant_12w		: std_logic; 

-------------------------
--signals from ruter 12--
-------------------------
signal x_out_12n_to_x_in_02s	: std_logic_vector(7 downto 0);
signal x_out_12w_to_x_in_11e	: std_logic_vector(7 downto 0);
signal x_out_12s_to_x_in_22n	: std_logic_vector(7 downto 0);
signal x_out_12e_to_x_in_13w	: std_logic_vector(7 downto 0);

signal RTS_12n_to_request_02s	: std_logic;
signal RTS_12w_to_request_11e	: std_logic;
signal RTS_12s_to_request_22n	: std_logic;
signal RTS_12e_to_request_13w	: std_logic;

signal CTS_12n_to_grant_02s		: std_logic;
signal CTS_12w_to_grant_11e		: std_logic;
signal CTS_12s_to_grant_22n		: std_logic;
signal CTS_12e_to_grant_13w		: std_logic;

-------------------------
--signals from ruter 13--
-------------------------
signal x_out_13n_to_x_in_03s	: std_logic_vector(7 downto 0);
signal x_out_13w_to_x_in_12e	: std_logic_vector(7 downto 0);
signal x_out_13s_to_x_in_23n	: std_logic_vector(7 downto 0);

signal RTS_13n_to_request_03s	: std_logic;
signal RTS_13w_to_request_12e	: std_logic;
signal RTS_13s_to_request_23n	: std_logic;

signal CTS_13n_to_grant_03s		: std_logic;
signal CTS_13w_to_grant_12e		: std_logic;
signal CTS_13s_to_grant_23n		: std_logic;

-------------------------
--signals from ruter 20--
-------------------------
signal x_out_20n_to_x_in_10s	: std_logic_vector(7 downto 0);
signal x_out_20s_to_x_in_30n	: std_logic_vector(7 downto 0);
signal x_out_20e_to_x_in_21w	: std_logic_vector(7 downto 0);

signal RTS_20n_to_request_10s	: std_logic;
signal RTS_20s_to_request_30n	: std_logic;
signal RTS_20e_to_request_21w	: std_logic;

signal CTS_20n_to_grant_10s		: std_logic;
signal CTS_20s_to_grant_30n		: std_logic;
signal CTS_20e_to_grant_21w		: std_logic;

-------------------------
--signals from ruter 21--
------------------------- 
signal x_out_21n_to_x_in_11s	: std_logic_vector(7 downto 0);
signal x_out_21w_to_x_in_20e	: std_logic_vector(7 downto 0);
signal x_out_21s_to_x_in_31n	: std_logic_vector(7 downto 0);
signal x_out_21e_to_x_in_22w	: std_logic_vector(7 downto 0);

signal RTS_21n_to_request_11s	: std_logic;
signal RTS_21w_to_request_20e	: std_logic;
signal RTS_21s_to_request_31n	: std_logic;
signal RTS_21e_to_request_22w	: std_logic;

signal CTS_21n_to_grant_11s		: std_logic;
signal CTS_21w_to_grant_20e		: std_logic;
signal CTS_21s_to_grant_31n		: std_logic;
signal CTS_21e_to_grant_22w		: std_logic; 

-------------------------
--signals from ruter 22--
-------------------------
signal x_out_22n_to_x_in_12s	: std_logic_vector(7 downto 0);
signal x_out_22w_to_x_in_21e	: std_logic_vector(7 downto 0);
signal x_out_22s_to_x_in_32n	: std_logic_vector(7 downto 0);
signal x_out_22e_to_x_in_23w	: std_logic_vector(7 downto 0);

signal RTS_22n_to_request_12s	: std_logic;
signal RTS_22w_to_request_21e	: std_logic;
signal RTS_22s_to_request_32n	: std_logic;
signal RTS_22e_to_request_23w	: std_logic;

signal CTS_22n_to_grant_12s		: std_logic;
signal CTS_22w_to_grant_21e		: std_logic;
signal CTS_22s_to_grant_32n		: std_logic;
signal CTS_22e_to_grant_23w		: std_logic;

-------------------------
--signals from ruter 23--
-------------------------
signal x_out_23n_to_x_in_13s	: std_logic_vector(7 downto 0);
signal x_out_23w_to_x_in_22e	: std_logic_vector(7 downto 0);
signal x_out_23s_to_x_in_33n	: std_logic_vector(7 downto 0);

signal RTS_23n_to_request_13s	: std_logic;
signal RTS_23w_to_request_22e	: std_logic;
signal RTS_23s_to_request_33n	: std_logic;

signal CTS_23n_to_grant_13s		: std_logic;
signal CTS_23w_to_grant_22e		: std_logic;
signal CTS_23s_to_grant_33n		: std_logic;

-------------------------
--signals from ruter 30--
-------------------------
signal x_out_30n_to_x_in_20s	: std_logic_vector(7 downto 0);
signal x_out_30e_to_x_in_31w	: std_logic_vector(7 downto 0);

signal RTS_30n_to_request_20s	: std_logic;
signal RTS_30e_to_request_31w	: std_logic;

signal CTS_30n_to_grant_20s		: std_logic;
signal CTS_30e_to_grant_31w		: std_logic;

-------------------------
--signals from ruter 31--
------------------------- 
signal x_out_31n_to_x_in_21s	: std_logic_vector(7 downto 0);
signal x_out_31w_to_x_in_30e	: std_logic_vector(7 downto 0);
signal x_out_31e_to_x_in_32w	: std_logic_vector(7 downto 0);

signal RTS_31n_to_request_21s	: std_logic;
signal RTS_31w_to_request_30e	: std_logic;
signal RTS_31e_to_request_32w	: std_logic;

signal CTS_31n_to_grant_21s		: std_logic;
signal CTS_31w_to_grant_30e		: std_logic;
signal CTS_31e_to_grant_32w		: std_logic; 

-------------------------
--signals from ruter 32--
-------------------------
signal x_out_32n_to_x_in_22s	: std_logic_vector(7 downto 0);
signal x_out_32w_to_x_in_31e	: std_logic_vector(7 downto 0);
signal x_out_32e_to_x_in_33w	: std_logic_vector(7 downto 0);

signal RTS_32n_to_request_22s	: std_logic;
signal RTS_32w_to_request_31e	: std_logic;
signal RTS_32e_to_request_33w	: std_logic;

signal CTS_32n_to_grant_22s		: std_logic;
signal CTS_32w_to_grant_31e		: std_logic;
signal CTS_32e_to_grant_33w		: std_logic;

-------------------------
--signals from ruter 33--
-------------------------
signal x_out_33n_to_x_in_23s	: std_logic_vector(7 downto 0);
signal x_out_33w_to_x_in_32e	: std_logic_vector(7 downto 0);

signal RTS_33n_to_request_23s	: std_logic;
signal RTS_33w_to_request_32e	: std_logic;

signal CTS_33n_to_grant_23s		: std_logic;
signal CTS_33w_to_grant_32e		: std_logic;

begin

             
ruter_00:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk,
	x_in_n      => d00,
	x_in_w      => d01,
	x_in_s      => x_out_10n_to_x_in_00s,
	x_in_e      => x_out_01w_to_x_in_00e,
	x_in_l      => x_in_l_00,
	request_n	=> dummy_CTS_RTS(0),
	request_w	=> dummy_CTS_RTS(1),
	request_s	=> RTS_10n_to_request_00s,
	request_e	=> RTS_01w_to_request_00e,
	request_l	=> request_l_00,
	grant_n     => dummy_CTS_RTS(2),
	grant_w     => dummy_CTS_RTS(3),
	grant_s     => CTS_10n_to_grant_00s,
	grant_e     => CTS_01w_to_grant_00e,
	grant_l		=> grant_l_00,
	x_out_n     => d02,
	x_out_w     => d03,
	x_out_s     => x_out_00s_to_x_in_10n,
	x_out_e     => x_out_00e_to_x_in_01w,
	x_out_l     => x_out_l_00,
	RTS_n       => dummy_CTS_RTS(4),
	RTS_w       => dummy_CTS_RTS(5),
	RTS_s       => RTS_00s_to_request_10n,
	RTS_e       => RTS_00e_to_request_01w,
	RTS_l	    => RTS_l_00,
	CTS_n       => dummy_CTS_RTS(6),
	CTS_w       => dummy_CTS_RTS(7),
	CTS_s       => CTS_00s_to_grant_10n,
	CTS_e       => CTS_00e_to_grant_01w,
	CTS_l       => CTS_l_00,
	coord       => coord00,
	PacketTrigger	=> PacketTrigger_00
	);
	
ruter_01:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => d04,
	x_in_w      => x_out_00e_to_x_in_01w,
	x_in_s      => x_out_11n_to_x_in_01s,
	x_in_e      => x_out_02w_to_x_in_01e,
	x_in_l      => x_in_l_01,
	request_n	=> dummy_CTS_RTS(8),
	request_w	=> RTS_00e_to_request_01w,
	request_s	=> RTS_11n_to_request_01s,
	request_e	=> RTS_02w_to_request_01e,
	request_l	=> request_l_01,
	grant_n     => dummy_CTS_RTS(9),
	grant_w     => CTS_00e_to_grant_01w,
	grant_s     => CTS_11n_to_grant_01s,
	grant_e     => CTS_02w_to_grant_01e,
	grant_l		=> grant_l_01,
	x_out_n     => d05,
	x_out_w     => x_out_01w_to_x_in_00e,
	x_out_s     => x_out_01s_to_x_in_11n,
	x_out_e     => x_out_01e_to_x_in_02w,
	x_out_l     => x_out_l_01,
	RTS_n       => dummy_CTS_RTS(10),
	RTS_w       => RTS_01w_to_request_00e,
	RTS_s       => RTS_01s_to_request_11n,
	RTS_e       => RTS_01e_to_request_02w,
	RTS_l	    => RTS_l_01,
	CTS_n       => dummy_CTS_RTS(11),
	CTS_w       => CTS_01w_to_grant_00e,
	CTS_s       => CTS_01s_to_grant_11n,
	CTS_e       => CTS_01e_to_grant_02w,
	CTS_l       => CTS_l_01,
	coord       => coord01,
	PacketTrigger	=> PacketTrigger_01
	);
	
ruter_02:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk,
	x_in_n      => d06,
	x_in_w      => x_out_01e_to_x_in_02w,
	x_in_s      => x_out_12n_to_x_in_02s,
	x_in_e      => x_out_03w_to_x_in_02e,
	x_in_l      => x_in_l_02,
	request_n	=> dummy_CTS_RTS(12),
	request_w	=> RTS_01e_to_request_02w,
	request_s	=> RTS_12n_to_request_02s,
	request_e	=> RTS_03w_to_request_02e,
	request_l	=> request_l_02,
	grant_n     => dummy_CTS_RTS(13),
	grant_w     => CTS_01e_to_grant_02w,
	grant_s     => CTS_12n_to_grant_02s,
	grant_e     => CTS_03w_to_grant_02e,
	grant_l		=> grant_l_02,
	x_out_n     => d07,
	x_out_w     => x_out_02w_to_x_in_01e,
	x_out_s     => x_out_02s_to_x_in_12n,
	x_out_e     => x_out_02e_to_x_in_03w,
	x_out_l     => x_out_l_02,
	RTS_n       => dummy_CTS_RTS(14),
	RTS_w       => RTS_02w_to_request_01e,
	RTS_s       => RTS_02s_to_request_12n,
	RTS_e       => RTS_02e_to_request_03w,
	RTS_l	    => RTS_l_02,
	CTS_n       => dummy_CTS_RTS(15),
	CTS_w       => CTS_02w_to_grant_01e,
	CTS_s       => CTS_02s_to_grant_12n,
	CTS_e       => CTS_02e_to_grant_03w,
	CTS_l       => CTS_l_02,
	coord       => coord02,
	PacketTrigger	=> PacketTrigger_02
	);

ruter_03:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => d08,
	x_in_w      => x_out_02e_to_x_in_03w,
	x_in_s      => x_out_13n_to_x_in_03s,
	x_in_e      => d09,
	x_in_l      => x_in_l_03,
	request_n	=> dummy_CTS_RTS(16),
	request_w	=> RTS_02e_to_request_03w,
	request_s	=> RTS_13n_to_request_03s,
	request_e	=> dummy_CTS_RTS(17),
	request_l	=> request_l_03,
	grant_n     => dummy_CTS_RTS(18),
	grant_w     => CTS_02e_to_grant_03w,
	grant_s     => CTS_13n_to_grant_03s,
	grant_e     => dummy_CTS_RTS(19),
	grant_l		=> grant_l_03,
	x_out_n     => d10,
	x_out_w     => x_out_03w_to_x_in_02e,
	x_out_s     => x_out_03s_to_x_in_13n,
	x_out_e     => d11,
	x_out_l     => x_out_l_03,
	RTS_n       => dummy_CTS_RTS(20),
	RTS_w       => RTS_03w_to_request_02e,
	RTS_s       => RTS_03s_to_request_13n,
	RTS_e       => dummy_CTS_RTS(21),
	RTS_l	    => RTS_l_03,
	CTS_n       => dummy_CTS_RTS(22),
	CTS_w       => CTS_03w_to_grant_02e,
	CTS_s       => CTS_03s_to_grant_13n,
	CTS_e       => dummy_CTS_RTS(23),
	CTS_l       => CTS_l_03,
	coord       => coord03,
	PacketTrigger	=> PacketTrigger_03
	);
	
ruter_10:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_00s_to_x_in_10n,
	x_in_w      => d12,
	x_in_s      => x_out_20n_to_x_in_10s,
	x_in_e      => x_out_11w_to_x_in_10e,
	x_in_l      => x_in_l_10,
	request_n	=> RTS_00s_to_request_10n,
	request_w	=> dummy_CTS_RTS(24),
	request_s	=> RTS_20n_to_request_10s,
	request_e	=> RTS_11w_to_request_10e,
	request_l	=> request_l_10,
	grant_n     => CTS_00s_to_grant_10n,
	grant_w     => dummy_CTS_RTS(25),
	grant_s     => CTS_20n_to_grant_10s,
	grant_e     => CTS_11w_to_grant_10e,
	grant_l		=> grant_l_10,
	x_out_n     => x_out_10n_to_x_in_00s,
	x_out_w     => d13,
	x_out_s     => x_out_10s_to_x_in_20n,
	x_out_e     => x_out_10e_to_x_in_11w,
	x_out_l     => x_out_l_10,
	RTS_n       => RTS_10n_to_request_00s,
	RTS_w       => dummy_CTS_RTS(26),
	RTS_s       => RTS_10s_to_request_20n,
	RTS_e       => RTS_10e_to_request_11w,
	RTS_l	    => RTS_l_10,
	CTS_n       => CTS_10n_to_grant_00s,
	CTS_w       => dummy_CTS_RTS(27),
	CTS_s       => CTS_10s_to_grant_20n,
	CTS_e       => CTS_10e_to_grant_11w,
	CTS_l       => CTS_l_10,
	coord       => coord10,
	PacketTrigger	=> PacketTrigger_10
	);
	
ruter_11:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_01s_to_x_in_11n,
	x_in_w      => x_out_10e_to_x_in_11w,
	x_in_s      => x_out_21n_to_x_in_11s,
	x_in_e      => x_out_12w_to_x_in_11e,
	x_in_l      => x_in_l_11,
	request_n	=> RTS_01s_to_request_11n,
	request_w	=> RTS_10e_to_request_11w,
	request_s	=> RTS_21n_to_request_11s,
	request_e	=> RTS_12w_to_request_11e,
	request_l	=> request_l_11,
	grant_n     => CTS_01s_to_grant_11n,
	grant_w     => CTS_10e_to_grant_11w,
	grant_s     => CTS_21n_to_grant_11s,
	grant_e     => CTS_12w_to_grant_11e,
	grant_l		=> grant_l_11,
	x_out_n     => x_out_11n_to_x_in_01s,
	x_out_w     => x_out_11w_to_x_in_10e,
	x_out_s     => x_out_11s_to_x_in_21n,
	x_out_e     => x_out_11e_to_x_in_12w,
	x_out_l     => x_out_l_11,
	RTS_n       => RTS_11n_to_request_01s,
	RTS_w       => RTS_11w_to_request_10e,
	RTS_s       => RTS_11s_to_request_21n,
	RTS_e       => RTS_11e_to_request_12w,
	RTS_l	    => RTS_l_11,
	CTS_n       => CTS_11n_to_grant_01s,
	CTS_w       => CTS_11w_to_grant_10e,
	CTS_s       => CTS_11s_to_grant_21n,
	CTS_e       => CTS_11e_to_grant_12w,
	CTS_l       => CTS_l_11,
	coord       => coord11,
	PacketTrigger	=> PacketTrigger_11
	);
	
ruter_12:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_02s_to_x_in_12n,
	x_in_w      => x_out_11e_to_x_in_12w,
	x_in_s      => x_out_22n_to_x_in_12s,
	x_in_e      => x_out_13w_to_x_in_12e,
	x_in_l      => x_in_l_12,
	request_n	=> RTS_02s_to_request_12n,
	request_w	=> RTS_11e_to_request_12w,
	request_s	=> RTS_22n_to_request_12s,
	request_e	=> RTS_13w_to_request_12e,
	request_l	=> request_l_12,
	grant_n     => CTS_02s_to_grant_12n,
	grant_w     => CTS_11e_to_grant_12w,
	grant_s     => CTS_22n_to_grant_12s,
	grant_e     => CTS_13w_to_grant_12e,
	grant_l		=> grant_l_12,
	x_out_n     => x_out_12n_to_x_in_02s,
	x_out_w     => x_out_12w_to_x_in_11e,
	x_out_s     => x_out_12s_to_x_in_22n,
	x_out_e     => x_out_12e_to_x_in_13w,
	x_out_l     => x_out_l_12,
	RTS_n       => RTS_12n_to_request_02s,
	RTS_w       => RTS_12w_to_request_11e,
	RTS_s       => RTS_12s_to_request_22n,
	RTS_e       => RTS_12e_to_request_13w,
	RTS_l	    => RTS_l_12,
	CTS_n       => CTS_12n_to_grant_02s,
	CTS_w       => CTS_12w_to_grant_11e,
	CTS_s       => CTS_12s_to_grant_22n,
	CTS_e       => CTS_12e_to_grant_13w,
	CTS_l       => CTS_l_12,
	coord       => coord12,
	PacketTrigger	=> PacketTrigger_12
	);

ruter_13:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_03s_to_x_in_13n,
	x_in_w      => x_out_12e_to_x_in_13w,
	x_in_s      => x_out_23n_to_x_in_13s,
	x_in_e      => d14,
	x_in_l      => x_in_l_13,
	request_n	=> RTS_03s_to_request_13n,
	request_w	=> RTS_12e_to_request_13w,
	request_s	=> RTS_23n_to_request_13s,
	request_e	=> dummy_CTS_RTS(28),
	request_l	=> request_l_13,
	grant_n     => CTS_03s_to_grant_13n,
	grant_w     => CTS_12e_to_grant_13w,
	grant_s     => CTS_23n_to_grant_13s,
	grant_e     => dummy_CTS_RTS(29),
	grant_l		=> grant_l_13,
	x_out_n     => x_out_13n_to_x_in_03s,
	x_out_w     => x_out_13w_to_x_in_12e,
	x_out_s     => x_out_13s_to_x_in_23n,
	x_out_e     => d15,
	x_out_l     => x_out_l_13,
	RTS_n       => RTS_13n_to_request_03s,
	RTS_w       => RTS_13w_to_request_12e,
	RTS_s       => RTS_13s_to_request_23n,
	RTS_e       => dummy_CTS_RTS(30),
	RTS_l	    => RTS_l_13,
	CTS_n       => CTS_13n_to_grant_03s,
	CTS_w       => CTS_13w_to_grant_12e,
	CTS_s       => CTS_13s_to_grant_23n,
	CTS_e       => dummy_CTS_RTS(31),
	CTS_l       => CTS_l_13,
	coord       => coord13,
	PacketTrigger	=> PacketTrigger_13
	);
	
ruter_20:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_10s_to_x_in_20n,
	x_in_w      => d16,
	x_in_s      => x_out_30n_to_x_in_20s,
	x_in_e      => x_out_21w_to_x_in_20e,
	x_in_l      => x_in_l_20,
	request_n	=> RTS_10s_to_request_20n,
	request_w	=> dummy_CTS_RTS(32),
	request_s	=> RTS_30n_to_request_20s,
	request_e	=> RTS_21w_to_request_20e,
	request_l	=> request_l_20,
	grant_n     => CTS_10s_to_grant_20n,
	grant_w     => dummy_CTS_RTS(33),
	grant_s     => CTS_30n_to_grant_20s,
	grant_e     => CTS_21w_to_grant_20e,
	grant_l		=> grant_l_20,
	x_out_n     => x_out_20n_to_x_in_10s,
	x_out_w     => d17,
	x_out_s     => x_out_20s_to_x_in_30n,
	x_out_e     => x_out_20e_to_x_in_21w,
	x_out_l     => x_out_l_20,
	RTS_n       => RTS_20n_to_request_10s,
	RTS_w       => dummy_CTS_RTS(34),
	RTS_s       => RTS_20s_to_request_30n,
	RTS_e       => RTS_20e_to_request_21w,
	RTS_l	    => RTS_l_20,
	CTS_n       => CTS_20n_to_grant_10s,
	CTS_w       => dummy_CTS_RTS(35),
	CTS_s       => CTS_20s_to_grant_30n,
	CTS_e       => CTS_20e_to_grant_21w,
	CTS_l       => CTS_l_20,
	coord       => coord20,
	PacketTrigger	=> PacketTrigger_20
	);
	
ruter_21:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_11s_to_x_in_21n,
	x_in_w      => x_out_20e_to_x_in_21w,
	x_in_s      => x_out_31n_to_x_in_21s,
	x_in_e      => x_out_22w_to_x_in_21e,
	x_in_l      => x_in_l_21,
	request_n	=> RTS_11s_to_request_21n,
	request_w	=> RTS_20e_to_request_21w,
	request_s	=> RTS_31n_to_request_21s,
	request_e	=> RTS_22w_to_request_21e,
	request_l	=> request_l_21,
	grant_n     => CTS_11s_to_grant_21n,
	grant_w     => CTS_20e_to_grant_21w,
	grant_s     => CTS_31n_to_grant_21s,
	grant_e     => CTS_22w_to_grant_21e,
	grant_l		=> grant_l_21,
	x_out_n     => x_out_21n_to_x_in_11s,
	x_out_w     => x_out_21w_to_x_in_20e,
	x_out_s     => x_out_21s_to_x_in_31n,
	x_out_e     => x_out_21e_to_x_in_22w,
	x_out_l     => x_out_l_21,
	RTS_n       => RTS_21n_to_request_11s,
	RTS_w       => RTS_21w_to_request_20e,
	RTS_s       => RTS_21s_to_request_31n,
	RTS_e       => RTS_21e_to_request_22w,
	RTS_l	    => RTS_l_21,
	CTS_n       => CTS_21n_to_grant_11s,
	CTS_w       => CTS_21w_to_grant_20e,
	CTS_s       => CTS_21s_to_grant_31n,
	CTS_e       => CTS_21e_to_grant_22w,
	CTS_l       => CTS_l_21,
	coord       => coord21,
	PacketTrigger	=> PacketTrigger_21
	);
	
ruter_22:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_12s_to_x_in_22n,
	x_in_w      => x_out_21e_to_x_in_22w,
	x_in_s      => x_out_32n_to_x_in_22s,
	x_in_e      => x_out_23w_to_x_in_22e,
	x_in_l      => x_in_l_22,
	request_n	=> RTS_12s_to_request_22n,
	request_w	=> RTS_21e_to_request_22w,
	request_s	=> RTS_32n_to_request_22s,
	request_e	=> RTS_23w_to_request_22e,
	request_l	=> request_l_22,
	grant_n     => CTS_12s_to_grant_22n,
	grant_w     => CTS_21e_to_grant_22w,
	grant_s     => CTS_32n_to_grant_22s,
	grant_e     => CTS_23w_to_grant_22e,
	grant_l		=> grant_l_22,
	x_out_n     => x_out_22n_to_x_in_12s,
	x_out_w     => x_out_22w_to_x_in_21e,
	x_out_s     => x_out_22s_to_x_in_32n,
	x_out_e     => x_out_22e_to_x_in_23w,
	x_out_l     => x_out_l_22,
	RTS_n       => RTS_22n_to_request_12s,
	RTS_w       => RTS_22w_to_request_21e,
	RTS_s       => RTS_22s_to_request_32n,
	RTS_e       => RTS_22e_to_request_23w,
	RTS_l	    => RTS_l_22,
	CTS_n       => CTS_22n_to_grant_12s,
	CTS_w       => CTS_22w_to_grant_21e,
	CTS_s       => CTS_22s_to_grant_32n,
	CTS_e       => CTS_22e_to_grant_23w,
	CTS_l       => CTS_l_22,
	coord       => coord22,
	PacketTrigger	=> PacketTrigger_22
	);

ruter_23:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_13s_to_x_in_23n,
	x_in_w      => x_out_22e_to_x_in_23w,
	x_in_s      => x_out_33n_to_x_in_23s,
	x_in_e      => d18,
	x_in_l      => x_in_l_23,
	request_n	=> RTS_13s_to_request_23n,
	request_w	=> RTS_22e_to_request_23w,
	request_s	=> RTS_33n_to_request_23s,
	request_e	=> dummy_CTS_RTS(36),
	request_l	=> request_l_23,
	grant_n     => CTS_13s_to_grant_23n,
	grant_w     => CTS_22e_to_grant_23w,
	grant_s     => CTS_33n_to_grant_23s,
	grant_e     => dummy_CTS_RTS(37),
	grant_l		=> grant_l_23,
	x_out_n     => x_out_23n_to_x_in_13s,
	x_out_w     => x_out_23w_to_x_in_22e,
	x_out_s     => x_out_23s_to_x_in_33n,
	x_out_e     => d19,
	x_out_l     => x_out_l_23,
	RTS_n       => RTS_23n_to_request_13s,
	RTS_w       => RTS_23w_to_request_22e,
	RTS_s       => RTS_23s_to_request_33n,
	RTS_e       => dummy_CTS_RTS(38),
	RTS_l	    => RTS_l_23,
	CTS_n       => CTS_23n_to_grant_13s,
	CTS_w       => CTS_23w_to_grant_22e,
	CTS_s       => CTS_23s_to_grant_33n,
	CTS_e       => dummy_CTS_RTS(39),
	CTS_l       => CTS_l_23,
	coord       => coord23,
	PacketTrigger	=> PacketTrigger_23
	); 
	
ruter_30:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_20s_to_x_in_30n,
	x_in_w      => d20,
	x_in_s      => d21,
	x_in_e      => x_out_31w_to_x_in_30e,
	x_in_l      => x_in_l_30,
	request_n	=> RTS_20s_to_request_30n,
	request_w	=> dummy_CTS_RTS(40),
	request_s	=> dummy_CTS_RTS(41),
	request_e	=> RTS_31w_to_request_30e,
	request_l	=> request_l_30,
	grant_n     => CTS_20s_to_grant_30n,
	grant_w     => dummy_CTS_RTS(42),
	grant_s     => dummy_CTS_RTS(43),
	grant_e     => CTS_31w_to_grant_30e,
	grant_l		=> grant_l_30,
	x_out_n     => x_out_30n_to_x_in_20s,
	x_out_w     => d22,
	x_out_s     => d23,
	x_out_e     => x_out_30e_to_x_in_31w,
	x_out_l     => x_out_l_30,
	RTS_n       => RTS_30n_to_request_20s,
	RTS_w       => dummy_CTS_RTS(44),
	RTS_s       => dummy_CTS_RTS(45),
	RTS_e       => RTS_30e_to_request_31w,
	RTS_l	    => RTS_l_30,
	CTS_n       => CTS_30n_to_grant_20s,
	CTS_w       => dummy_CTS_RTS(46),
	CTS_s       => dummy_CTS_RTS(47),
	CTS_e       => CTS_30e_to_grant_31w,
	CTS_l       => CTS_l_30,
	coord       => coord30,
	PacketTrigger	=> PacketTrigger_30
	);
	
ruter_31:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_21s_to_x_in_31n,
	x_in_w      => x_out_30e_to_x_in_31w,
	x_in_s      => d24,
	x_in_e      => x_out_32w_to_x_in_31e,
	x_in_l      => x_in_l_31,
	request_n	=> RTS_21s_to_request_31n,
	request_w	=> RTS_30e_to_request_31w,
	request_s	=> dummy_CTS_RTS(48),
	request_e	=> RTS_32w_to_request_31e,
	request_l	=> request_l_31,
	grant_n     => CTS_21s_to_grant_31n,
	grant_w     => CTS_30e_to_grant_31w,
	grant_s     => dummy_CTS_RTS(49),
	grant_e     => CTS_32w_to_grant_31e,
	grant_l		=> grant_l_31,
	x_out_n     => x_out_31n_to_x_in_21s,
	x_out_w     => x_out_31w_to_x_in_30e,
	x_out_s     => d25,
	x_out_e     => x_out_31e_to_x_in_32w,
	x_out_l     => x_out_l_31,
	RTS_n       => RTS_31n_to_request_21s,
	RTS_w       => RTS_31w_to_request_30e,
	RTS_s       => dummy_CTS_RTS(50),
	RTS_e       => RTS_31e_to_request_32w,
	RTS_l	    => RTS_l_31,
	CTS_n       => CTS_31n_to_grant_21s,
	CTS_w       => CTS_31w_to_grant_30e,
	CTS_s       => dummy_CTS_RTS(51),
	CTS_e       => CTS_31e_to_grant_32w,
	CTS_l       => CTS_l_31,
	coord       => coord31,
	PacketTrigger	=> PacketTrigger_31
	);
	
ruter_32:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_22s_to_x_in_32n,
	x_in_w      => x_out_31e_to_x_in_32w,
	x_in_s      => d26,
	x_in_e      => x_out_33w_to_x_in_32e,
	x_in_l      => x_in_l_32,
	request_n	=> RTS_22s_to_request_32n,
	request_w	=> RTS_31e_to_request_32w,
	request_s	=> dummy_CTS_RTS(52),
	request_e	=> RTS_33w_to_request_32e,
	request_l	=> request_l_32,
	grant_n     => CTS_22s_to_grant_32n,
	grant_w     => CTS_31e_to_grant_32w,
	grant_s     => dummy_CTS_RTS(53),
	grant_e     => CTS_33w_to_grant_32e,
	grant_l		=> grant_l_32,
	x_out_n     => x_out_32n_to_x_in_22s,
	x_out_w     => x_out_32w_to_x_in_31e,
	x_out_s     => d27,
	x_out_e     => x_out_32e_to_x_in_33w,
	x_out_l     => x_out_l_32,
	RTS_n       => RTS_32n_to_request_22s,
	RTS_w       => RTS_32w_to_request_31e,
	RTS_s       => dummy_CTS_RTS(54),
	RTS_e       => RTS_32e_to_request_33w,
	RTS_l	    	=> RTS_l_32,
	CTS_n       => CTS_32n_to_grant_22s,
	CTS_w       => CTS_32w_to_grant_31e,
	CTS_s       => dummy_CTS_RTS(55),
	CTS_e       => CTS_32e_to_grant_33w,
	CTS_l       => CTS_l_32,
	coord       => coord32,
	PacketTrigger	=> PacketTrigger_32
	);

ruter_33:entity noc_v1_00_a.router(behavioral)
	port map(
	reset       => reset,
	clk         => clk, 
	x_in_n      => x_out_23s_to_x_in_33n,
	x_in_w      => x_out_32e_to_x_in_33w,
	x_in_s      => d28,
	x_in_e      => d29,
	x_in_l      => x_in_l_33,
	request_n	=> RTS_23s_to_request_33n,
	request_w	=> RTS_32e_to_request_33w,
	request_s	=> dummy_CTS_RTS(56),
	request_e	=> dummy_CTS_RTS(57),
	request_l	=> request_l_33,
	grant_n     => CTS_23s_to_grant_33n,
	grant_w     => CTS_32e_to_grant_33w,
	grant_s     => dummy_CTS_RTS(58),
	grant_e     => dummy_CTS_RTS(59),
	grant_l		=> grant_l_33,
	x_out_n     => x_out_33n_to_x_in_23s,
	x_out_w     => x_out_33w_to_x_in_32e,
	x_out_s     => d30,
	x_out_e     => d31,
	x_out_l     => x_out_l_33,
	RTS_n       => RTS_33n_to_request_23s,
	RTS_w       => RTS_33w_to_request_32e,
	RTS_s       => dummy_CTS_RTS(60),
	RTS_e       => dummy_CTS_RTS(61),
	RTS_l	    	=> RTS_l_33,
	CTS_n       => CTS_33n_to_grant_23s,
	CTS_w       => CTS_33w_to_grant_32e,
	CTS_s       => dummy_CTS_RTS(62),
	CTS_e       => dummy_CTS_RTS(63),
	CTS_l       => CTS_l_33,
	coord       => coord33,
	PacketTrigger	=> PacketTrigger_33
	);


        ---------------------
        -- Set coordinates --
	process (reset) is        
	begin
        
    if reset = '0' then
        coord00 <= "0000";        
        coord01 <= "0001";
        coord02 <= "0010";
        coord03 <= "0011";
        
        coord10 <= "0100";
        coord11 <= "0101";
        coord12 <= "0110";
        coord13 <= "0111";
        
        coord20 <= "1000";
        coord21 <= "1001";
        coord22 <= "1010";
        coord23 <= "1011";
        
        coord30 <= "1100";
        coord31 <= "1101";
        coord32 <= "1110";
        coord33 <= "1111";
        
		dummy_CTS_RTS <= (others=> '0');
		
	end if;
	end process;
end architecture;
