library ieee;
use ieee.std_logic_1164.all;

entity arbiter_v2 is
	port(
	clk			: in std_logic;
	reset		: in std_logic;
	x_in_n  	: in std_logic_vector(7 downto 0);
	x_in_w     	: in std_logic_vector(7 downto 0);
	x_in_s     	: in std_logic_vector(7 downto 0);
	x_in_e     	: in std_logic_vector(7 downto 0);
	x_in_l     	: in std_logic_vector(7 downto 0); 
	request_n	: in std_logic;
	request_w	: in std_logic;
	request_s	: in std_logic;
	request_e	: in std_logic;
	request_l	: in std_logic;
	grant_n     : in std_logic;
	grant_w     : in std_logic;
	grant_s     : in std_logic;
	grant_e     : in std_logic;
	grant_l	   	: in std_logic;
	packet_read : in std_logic;
	granted		: out std_logic;
	request_ctrl: out std_logic;
	serv_arb	: out std_logic_vector(2 downto 0);
	x_in		: out std_logic_vector(7 downto 0) 
	);											  
end entity;

architecture behavioral of arbiter_v2 is

signal lreq_n, lreq_w, lreq_s, lreq_e, lreq_l	: std_logic;
signal serving              					: std_logic_vector(2 downto 0);

begin
	serv_arb <= serving;
	granted <= grant_n or grant_w or grant_s or grant_e or grant_l;
	
	with serving select
	x_in <= x_in_n when "000",
	    	x_in_w when "001",
	        x_in_s when "010",
	        x_in_e when "011",
	        x_in_l when others;
			
process(clk,reset) is

variable arbitrate : std_logic;

begin
	if reset = '0' then
		lreq_n <= '0';
		lreq_w <= '0';
		lreq_s <= '0';
		lreq_e <= '0';
		lreq_l <= '0';
		request_ctrl <= '0';
		serving <= "000";	   
		arbitrate := '1';
		
	elsif clk'event and clk = '1' then
		if arbitrate = '1' then
			if request_n = '1' and lreq_n = '0' then
				serving <= "000";
				lreq_n <= '1';
				lreq_w <= '0';
				lreq_s <= '0';
				lreq_e <= '0';
				lreq_l <= '0';
				arbitrate := '0';
				
			elsif request_w = '1' and lreq_w = '0' then
				serving <= "001";
				lreq_n <= '0';
				lreq_w <= '1';
				lreq_s <= '0';
				lreq_e <= '0';
				lreq_l <= '0';
				arbitrate := '0';
				
			elsif request_s = '1' and lreq_s = '0' then
				serving <= "010";
				lreq_n <= '0';
				lreq_w <= '0';
				lreq_s <= '1';
				lreq_e <= '0';
				lreq_l <= '0';	 
				arbitrate := '0';
				
			elsif request_e = '1' and lreq_e = '0' then
				serving <= "011";
				lreq_n <= '0';
				lreq_w <= '0';
				lreq_s <= '0';
				lreq_e <= '1';
				lreq_l <= '0';	 
				arbitrate := '0';
				
			elsif request_l = '1' and lreq_l = '0' then
				serving <= "100";
				lreq_n <= '0';
				lreq_w <= '0';
				lreq_s <= '0';
				lreq_e <= '0';
				lreq_l <= '1';	 
				arbitrate := '0';
			else --no other requests, zero out every lreq_l
				lreq_n <= '0';
				lreq_w <= '0';
				lreq_s <= '0';
				lreq_e <= '0';
				lreq_l <= '0';
				
			end if;
		end if;
		
		if packet_read = '1' then
			arbitrate := '1';
		end if;		
		
		request_ctrl <=  request_n or request_w or request_s or request_e or request_l;
	end if;
end process;	
end architecture;	
		
			
		
		