--------------------------------------------------------------------------
-- The Control Module        2                                          --
--------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL; 
use IEEE.STD_LOGIC_ARITH.ALL; 
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity control2 is
    Port (  request 	:   in   STD_LOGIC;  					--from arbiter
            granted    	:   in   STD_LOGIC;						--from arbiter
            clk        	:   in   STD_LOGIC;
            reset      	:   in   STD_LOGIC;
            RTS        	:   out  STD_LOGIC;                     -- Request To Send
            CTS        	:   out  STD_LOGIC;                     -- Cleared to Send/Granted to Send
            instructA  	:   out  STD_LOGIC_VECTOR(1 downto 0);  -- Instruction for the buffer A
            instructB  	:   out  STD_LOGIC_VECTOR(1 downto 0);  -- Instruction for the buffer B          
            use_buff   	:   out  STD_LOGIC;					    -- to readout
            read       	:   out  STD_LOGIC;					-- to readout
			packet_read	: 	out std_logic
			);
end control2;

architecture behavioral of control2 is
	type InState is (IDLE, READ_IN);
	type OutState is (IDLE, HOLD, TRANSMIT);
	
	signal stateA,stateB                    : STD_LOGIC_VECTOR(1 downto 0);
    signal transmit_buff                    : STD_LOGIC := '0';
	signal counterA, counterB				: INTEGER range -3 to 8;   
	
    begin
    
	use_buff <= transmit_buff;
	
	---------------------------------------------------------
	--Muxing of the instructions for the Buffers, so 	  --
	--that the correct buffer gets the correct insturction.--
	---------------------------------------------------------
	with transmit_buff select
		instructA <= stateA when '0',
					 stateB when others;
	
	with transmit_buff select
		instructB <= stateB when '0',
					 stateA when others;   
			
	------------------------------------------------------------------------------					
	--Process which drives the two statemachines for the router input and output--
	------------------------------------------------------------------------------
    process(clk,reset) is							  
	
	variable trans_state, ntrans_state		: OutState;
	variable read_state, nread_state		: Instate;
    variable contains_packet                   : STD_LOGIC_VECTOR(1 downto 0);
    variable ntransmit_buff                    : STD_LOGIC;
	
	begin 
		if reset ='0' then
			nread_state 	:= IDLE;
			ntrans_state 	:= IDLE;
			ntransmit_buff 	:= '0';
			counterA 		<= -1; 
			counterB 		<= 0;
			RTS 			<= '0';
			CTS 			<= '0';
			contains_packet := "00";
			stateA 			<= "10"; --IDLE
			stateB 			<= "10"; --IDLE
			read 			<= '0';
			packet_read 	<= '0';
		
		elsif clk'event and clk='1' then
			
			------------------------------
			--STATEMACHINE FOR THE INPUT--
			------------------------------
			
			case read_state is
			when IDLE =>
				if request = '1' then
					if contains_packet /= "11" then	--Checking if there is possible to allocate one of the buffers.
						nread_state := READ_IN;
						CTS <= '1';
					end if;
				else
					nread_state := IDLE;
				end if;

			when READ_IN =>  	
            stateA(1) <= '0';
				CTS <= '1';
				if counterA = 7 then -- last flit is read, return to IDLE
					nread_state := IDLE;
					stateA(1) <= '1'; --IDLE
					CTS <= '0';
					counterA <= -1;
					read <= '0';
					packet_read <= '0';
					if transmit_buff = '1' then --Signal that the allocated buffer contains a packet ready for readout.
						contains_packet(0) := '1';
					else
						contains_packet(1) := '1';
					end if;
				
				else
					if counterA = 6 then
						CTS <= '0';
						packet_read <= '1';
					end if;
					counterA <= counterA + 1;
				end if;
			when others => null;
			end case;
			
			-------------------------------
			--STATEMACHINE FOR THE OUTPUT--
			-------------------------------
			
			case trans_state is
			when IDLE =>
			stateB <= "10"; --BUFFER_IDLE
				--Checks if there is any packages to send, and sends them if there is.
				if contains_packet(1) = '1' then
					ntrans_state := HOLD;
					ntransmit_buff := '1';
					read <= '1';
				elsif contains_packet(0) = '1' then
					ntrans_state := HOLD;
					ntransmit_buff := '0';
					read <= '1';
				end if;
	
			when HOLD =>
				stateB <= "10"; --IDLE 
				read <= '0'; --Reads the headerflits address needed for the XY-algorithm, performed in the Readout module
				RTS <= '1';
				--waiting for permission to send our package.
				if granted = '1' then --granted to send from destination 
					ntrans_state := TRANSMIT;
					stateB <= "01"; --BUFFER_READ_OUT
					counterB <= counterB + 1;		
				end if;
			
			when TRANSMIT =>
				if counterB = 7 then
					if granted = '1' then -- last flit transmitted, return to IDLE.
						ntrans_state := IDLE;
						counterB <= 0;
						RTS <= '0';
						if transmit_buff = '1' then	--signal that the package is transmittet and the buffer is now empty.
							contains_packet(1) := '0';
						else
							contains_packet(0) := '0';
						end if;             
					end if;
				else -- Transmitting
					counterB <= counterB + 1;
				end if;

			when others => null;
			end case;     
		
			read_state := nread_state; 
			trans_state := ntrans_state;
			transmit_buff <= ntransmit_buff;       
		end if;    
	end process;
end architecture;
