---------------------------------------------------------------------------------
-- Ruter ut signalene til 64 bit bufferet
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity readout  is
    Port ( x_out_n      :        out     STD_LOGIC_VECTOR(7 downto 0);
           x_out_w      :        out     STD_LOGIC_VECTOR(7 downto 0);
           x_out_s      :        out     STD_LOGIC_VECTOR(7 downto 0);
           x_out_e      :        out     STD_LOGIC_VECTOR(7 downto 0);
           x_out_l      :        out     STD_LOGIC_VECTOR(7 downto 0);
           RTS_n        :        out     STD_LOGIC;
           RTS_w        :        out     STD_LOGIC;
           RTS_s        :        out     STD_LOGIC;
           RTS_e        :        out     STD_LOGIC;
           RTS_l    	:        out     STD_LOGIC;
           CTS_n        :        out     STD_LOGIC;
           CTS_w        :        out     STD_LOGIC;
           CTS_s        :        out     STD_LOGIC;
           CTS_e        :        out     STD_LOGIC;
           CTS_l        :        out     STD_LOGIC;
           clk          :        in      STD_LOGIC;
           coord        :        in      STD_LOGIC_VECTOR(3 downto 0);
           serv         :        in      STD_LOGIC_VECTOR(2 downto 0);
           RTS_arb      :        in      STD_LOGIC;
           CTS_arb      :        in      STD_LOGIC;
           use_buff     :        in      STD_LOGIC;
           x_out_buffA  :        in      STD_LOGIC_VECTOR(7 downto 0);           
           x_out_buffB  :        in      STD_LOGIC_VECTOR(7 downto 0);                      
           reset        :        in      STD_LOGIC;
           adr_buffA    :        in      STD_LOGIC_VECTOR(3 downto 0);
           adr_buffB    :        in      STD_LOGIC_VECTOR(3 downto 0);
           read         :        in      STD_LOGIC);

end readout;

architecture Behavioral of readout is
    
    signal x_out        : STD_LOGIC_VECTOR(39 downto 0);
    signal RTS          : STD_LOGIC_VECTOR(4 downto 0);
    signal CTS          : STD_LOGIC_VECTOR(4 downto 0);
    signal destination  : STD_LOGIC_VECTOR(2 downto 0);
    signal x_out_buff   : STD_LOGIC_VECTOR(7 downto 0);
    signal adr_buff     : STD_LOGIC_VECTOR(3 downto 0);
    
begin
	
    x_out_n <= x_out(31 downto 24);
    x_out_w <= x_out(23 downto 16);
    x_out_s <= x_out(15 downto 8);
    x_out_e <= x_out(7 downto 0);
    x_out_l <= x_out(39 downto 32);
    
    RTS_n <= RTS(3);
    RTS_w <= RTS(2);
    RTS_s <= RTS(1);
    RTS_e <= RTS(0);
    RTS_l <= RTS(4);
    
    CTS_n <= CTS(3);
    CTS_w <= CTS(2);
    CTS_s <= CTS(1);
    CTS_e <= CTS(0);
    CTS_l <= CTS(4);
    
    with use_buff select
    adr_buff <= adr_buffA when '1',
                adr_buffB when others;
    
    with use_buff select
    x_out_buff <= x_out_buffA when '1',
                  x_out_buffB when others;
    
   -- demux to route flits      -- 
    with destination select
    x_out <= x"00" & x_out_buff & x"000000" when "000",
             x"0000" & x_out_buff & x"0000" when "001",
             x"000000" & x_out_buff & x"00" when "010",
             x"00000000" & x_out_buff       when "011",
             x_out_buff & x"00000000"       when "100",
			 x_out_buff & x"00000000"       when others;
    
    ---------------------------------
    -- demux to route Request      -- 
    with destination select
    RTS <= '0' & RTS_arb & "000" when "000",
           "00" & RTS_arb & "00" when "001",
           "000" & RTS_arb & '0' when "010",
           "0000" & RTS_arb      when "011",
           RTS_arb & "0000"      when "100",
		   RTS_arb & "0000"      when others;
              
              
    ---------------------------------------
    -- demux to rute Clear To Send       --
	---------------------------------------
	
    with serv select --originalt serv
    CTS <= '0' & CTS_arb & "000" when "000",
           "00" & CTS_arb & "00" when "001",
           "000" & CTS_arb & '0' when "010",
           "0000" & CTS_arb      when "011",
           CTS_arb & "0000"      when others;
              
    -----------------------------------------
    -- process to read destination address -- 
    -- and detrmine routing direction      --
	-----------------------------------------
	
    process (clk,reset) is
        variable adr               : STD_LOGIC_VECTOR(3 downto 0); -- 4 bit destination address     
        variable RTS_arb_prev      : STD_LOGIC;
    begin
    
        if reset = '0' then
			destination <= "000";
			adr := "0000";
        elsif clk'event and clk = '1' then
			
			if read = '1' then 				----------------------------------
			      adr := adr_buff;			-- Reads in the adress for the	--
			end if;							--package from the adress buffer--
			---------------------------		----------------------------------
			-- x-y routing algorithm --
			---------------------------
			
			if adr(1 downto 0) > coord(1 downto 0) then   	--Er desitnasjonens x-koordinat strre enn ruterens
			    destination <= "011"; 						--Destinasjonen er lik vest
			elsif adr(1 downto 0) < coord(1 downto 0) then	--Er destinasjonens x-koordinat mindre enn ruterens
			    destination <= "001";						--Destinasjonen er lik st
			else											--Hvis pakken har korrekt x-koordinat
			    if adr(3 downto 2) > coord(3 downto 2) then		--Er destinasjonens y-koordinat strre enn ruterens
			          destination <= "010";						--Destinasjonen er lik sr
			    elsif adr(3 downto 2) < coord(3 downto 2) then	--Er destinasjonens y-koordinat mindre enn ruterens
			          destination <= "000";						--Destinasjonen er lik nord
			    else											--Hvis ruteren har korrekt x- og y-koordinat
			          destination <= "100";						--Destinasjonen er den lokale modulen
			    end if;                 
			end if;
			
			RTS_arb_prev := RTS_arb;
			
			end if;
			end process;                      
			end architecture;