library ieee;
use ieee.std_logic_1164.all;
    
entity router is
	port(  
		reset       :in std_logic;
		clk         :in std_logic;
		coord       :in std_logic_vector(3 downto 0);
		x_in_n      :in std_logic_vector(7 downto 0);
		x_in_w      :in std_logic_vector(7 downto 0);
		x_in_s      :in std_logic_vector(7 downto 0);
		x_in_e      :in std_logic_vector(7 downto 0);
		x_in_l      :in std_logic_vector(7 downto 0);
		request_n	:in std_logic;
		request_w	:in std_logic;
		request_s	:in std_logic;
		request_e	:in std_logic;
		request_l	:in std_logic;
		grant_n     :in std_logic;
		grant_w     :in std_logic;
		grant_s     :in std_logic;
		grant_e     :in std_logic;                     
		grant_l		:in std_logic;
		x_out_n     :out std_logic_vector(7 downto 0);
		x_out_w     :out std_logic_vector(7 downto 0);
		x_out_s     :out std_logic_vector(7 downto 0);
		x_out_e     :out std_logic_vector(7 downto 0); 
		x_out_l     :out std_logic_vector(7 downto 0);
		RTS_n       :out std_logic;
		RTS_w       :out std_logic;
		RTS_s       :out std_logic;
		RTS_e       :out std_logic;
		RTS_l	    :out std_logic;
		CTS_n       :out std_logic;
		CTS_w       :out std_logic;
		CTS_s       :out std_logic;
		CTS_e       :out std_logic;
		CTS_l       :out std_logic;
		PacketTrigger	:out std_logic
	);                
end entity;

      
architecture behavioral of router is

--Arbiter signals
signal request    	: std_logic;
signal serv_arb		: std_logic_vector(2 downto 0);
signal granted    	: std_logic;
signal x_in			: std_logic_vector(7 downto 0);

--Control signals
signal instructA  	: std_logic_vector(1 downto 0);
signal instructB  	: std_logic_vector(1 downto 0);
signal RTS		  	: std_logic;
signal CTS    	  	: std_logic;
signal use_buff	  	: std_logic;
signal read		  	: std_logic;
signal packet_read	: std_logic;

--Buffer signals
signal x_outA		: std_logic_vector(7 downto 0);	
signal x_outB 		: std_logic_vector(7 downto 0);      
signal adr_buffA	: std_logic_vector(3 downto 0);
signal adr_buffB	: std_logic_vector(3 downto 0);
	  
begin --- Node 00 ---

arb: entity noc_v1_00_a.arbiter_v2(behavioral)
	port map(
	x_in_n 		=> x_in_n,
	x_in_w 		=> x_in_w,
	x_in_s 		=> x_in_s,
	x_in_e 		=> x_in_e,
	x_in_l 		=> x_in_l,
	request_n 	=> request_n,
	request_w 	=> request_w,
	request_s 	=> request_s,
	request_e 	=> request_e,
	request_l 	=> request_l,
	grant_n 	=> grant_n,
	grant_w 	=> grant_w,
	grant_s 	=> grant_s,
	grant_e 	=> grant_e,
	grant_l 	=> grant_l,
	packet_read => packet_read,
	clk 		=> clk,
	reset 		=> reset,
	granted 	=> granted,
	request_ctrl=> request,
	serv_arb 	=> serv_arb,
	x_in 		=> x_in
	);
   
ctrl: entity noc_v1_00_a.control2(behavioral)
	port map(
	request		=> request, 
	granted     => granted,
	clk       	=> clk,
	reset     	=> reset,
	RTS        	=> RTS,
	CTS        	=> CTS,
	instructA  	=> instructA,
	instructB  	=> instructB,
	use_buff   	=> use_buff,
	read       	=> read,
	packet_read => packet_read
	);

buffA:   entity noc_v1_00_a.buffer_64bit(behavioral)
	port map(
	x_in        => x_in,
	instruction => instructA,
	clk         => clk,
	reset       => reset,
	x_out       => x_outA,
	adresse		=> adr_buffA
	);

buffB:   entity noc_v1_00_a.buffer_64bit(behavioral)
	port map(
	x_in        => x_in,
	instruction => instructB,
	clk         => clk,
	reset       => reset,
	x_out       => x_outB,
	adresse		=> adr_buffB
	);
	
readout: entity noc_v1_00_a.readout(behavioral)
	port map(
	x_out_n      => x_out_n,
	x_out_w      =>	x_out_w,
	x_out_s      =>	x_out_s,
	x_out_e      =>	x_out_e,
	x_out_l      =>	x_out_l,
	RTS_n        =>	RTS_n,
	RTS_w        =>	RTS_w,
	RTS_s        =>	RTS_s,
	RTS_e        =>	RTS_e,
	RTS_l	     =>	RTS_l,
	CTS_n        =>	CTS_n,
	CTS_w        =>	CTS_w,
	CTS_s        =>	CTS_s,
	CTS_e        =>	CTS_e,
	CTS_l        =>	CTS_l,
	clk          =>	clk,
	coord        =>	coord,
	serv         =>	serv_arb,
	RTS_arb      =>	RTS,
	CTS_arb      =>	CTS,
	use_buff     =>	use_buff,
	x_out_buffA  =>	x_outA,
	x_out_buffB  =>	x_outB,
	reset        =>	reset,
	adr_buffA    =>	adr_buffA,
	adr_buffB    =>	adr_buffB,
	read         =>	read
	);
	
	PacketTrigger <= CTS;
	
end architecture;