----------------------------------
--16 word by 16 bit BRAM module --
--Written by: Andreas Heps     --
----------------------------------
--Summary: Module that can store--
--16 std_logic_vectors of length--
--16.							--
----------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity BRAM is
	generic
	(
	C_SLV_AWIDTH                   : integer              := 32
	);
	port(
	ungated_clk	: in std_logic;
	clk			: in std_logic;
	enable		: in std_logic;
	SW_enable	: in std_logic;
	wenable		: in std_logic;
	addr		: in std_logic_vector(8 downto 0);
	SW_addr		: in std_logic_vector(8 downto 0);
	data_i		: in std_logic_vector(C_SLV_AWIDTH-1 downto 0);
	data_o		: out std_logic_vector(C_SLV_AWIDTH-1 downto 0)	
	);													   
end entity BRAM;

architecture behavioral of BRAM is
type BRAM_inst is array(0 to 511) of std_logic_vector(C_SLV_AWIDTH-1 downto 0);

signal BRAM	: BRAM_inst;

begin
	
	HW_PROC: process(clk)
	begin
		if(clk'event and clk = '1') then
			if(enable = '1') then
				if(wenable = '1') then
					BRAM(to_integer(UNSIGNED(addr))) <= data_i;
				end if;
			end if;
		end if;
	end process;
	
	SW_PROC: process(ungated_clk)
	begin
		if(ungated_clk'event and ungated_clk = '1') then
			if(SW_enable = '1') then
				data_o <= BRAM(to_integer(UNSIGNED(SW_addr)));
			end if;
		end if;
	end process;
end architecture;
