library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity TM_control is
	port(
	clk				: in std_logic;
	reset			: in std_logic;
	CLK_GATE_FREQ	: in std_logic_vector(3 downto 0);
	readout_finished: in std_logic;
	start_testing	: in std_logic;
	clock_enable	: out std_logic;
	readout			: out std_logic
	);
end entity;

architecture behavioral of TM_control is

signal count 					: integer range 0 to 100000;
signal internal_holt			: std_logic;
signal prev_readout_finished	: std_logic; --positive flank detektor signal

begin
	
	process(clk, reset, start_testing) is
	begin
		if reset = '0' then
			--reset all;
			count 					<= 1;
			readout 				<= '0';
			clock_enable			<= '1';
			internal_holt			<= '0';
			prev_readout_finished	<= '0';
			
		elsif clk'event and clk = '1' and start_testing = '1' then

			if count = 6144*to_integer(UNSIGNED(CLK_GATE_FREQ)) then
				--stop clock and init readout
				internal_holt 	<= '1';
				readout 		<= '1';
				clock_enable	<= '0';
				count			<= 0;
				--set readout = 0 when finished and activate the clock again.....
			elsif internal_holt = '0' then
				readout 		<= '0';
				clock_enable	<= '1';
				count 			<= count + 1;
			else
				null;
			end if;	 
			
			if readout_finished = '1' and prev_readout_finished = '0' then --SW signals that the data is stored.
				internal_holt <= '0';
			end if;
			
			prev_readout_finished <= readout_finished;
								
		end if;
		
	end process;
end architecture;
	