library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity TM is
port(	clk					: in std_logic;
		reset				: in std_logic;
		CTS					: in std_logic;
		readout_finished	: in std_logic;	--from global SW control unit
		packet_counter		: out integer
);
end entity;

architecture behavioral of TM is

signal CTS_prev					: std_logic; 	--edge detector signal
signal readout_finished_prev 	: std_logic;	--edge detector signal
signal internal_reset			: std_logic;

begin                     
	process(clk,reset,readout_finished)is
	
	variable packetCounter :integer;
	
	begin 	
	if reset = '0' then
		CTS_prev 				<= '0';
		readout_finished_prev 	<= '0';
		packet_counter 			<= 0;
		packetCounter 			:= 0;
		internal_reset			<= '1';	
			
	elsif clk'event and clk = '1' then
		if internal_reset = '0' then
			packet_counter			<= 0;
			packetCounter			:= 0;
			internal_reset			<= '1';
			
		elsif CTS = '1' and CTS_prev = '0' then
			packetCounter := packetCounter + 1;	
		end if;
		
		CTS_prev <= CTS;
		readout_finished_prev <= readout_finished;
		packet_counter <= packetCounter;
	end if;
	
	if readout_finished = '1' and readout_finished_prev = '0' then --SW has read the data
		internal_reset 		<= '0';
	end if;
	
	end process;	
end architecture;