------------------------------
-- Written by Andreas Heps --
------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity interface is
port( 	clk						: in  std_logic;
		reset         			: in  std_logic;
		data_in             	: in  std_logic_vector(63 downto 0); 	--Signal containing data from slv_reg0 and slv_reg1
		send_packet	        	: in  std_logic; 						--Signal which activates sending of information lying on data_in, accesed by slv_reg4(1)
		mux_select				: in  std_logic_vector(3 downto 0); 	--select signal for the mux.
		BRAM_enable				: in  std_logic;
		BRAM_addr				: in  std_logic_vector(8 downto 0);
		readout_finished		: in  std_logic; 						--From SW, stored all values
		CLK_GATE_FREQ			: in  std_logic_vector(3 downto 0);		--Set from SW, decides the clock gate frequency
		start_testing			: in  std_logic;						--Starts the TM_control module, and thus the data collecting.
		data_out          		: out std_logic_vector(63 downto 0); 	--Signal written to slv_reg2 and slv_reg3.
		packet_received_33 		: out std_logic; 						--Signal which informs that a package have arrived.
		packet_sendt			: out std_logic; 						--Signal which informs that a package has been sendt.
		mux_o_packetcnt			: out std_logic_vector(31 downto 0); 	--(MUX output) - Packet counters
		mux_o_tstamp			: out std_logic_vector(31 downto 0);
		readout_SW				: out std_logic							--readout to SW, to know the TM_control has stopped.
	 );
end entity;

architecture behavioral of interface is

type SendState is (IDLE, HOLD, SEND);
type RecvState is (IDLE, RECEIVE);

--declaring signals for statemachines
signal recv_counter				:INTEGER range -1 to 8;
signal send_counter 				:INTEGER range 0 to 8;
signal packet_in,packet_out	:std_logic_vector(63 downto 0);

--declaring signals for 44_interface_mesh
signal grant,request	:std_logic;
signal RTS,CTS			:std_logic;
signal x_in,x_out		:std_logic_vector(7 downto 0);
signal readout			:std_logic;
signal clock_enable		:std_logic;
signal i_clk			:std_logic;

signal prev_send_packet	:std_logic; --Flank detektor signal for send_packet

--Component declaration
component BUFGCE
	port (
	O 	: out STD_ULOGIC;
	CE : in STD_ULOGIC;
	I 	: in STD_ULOGIC);
end component;

begin
	
CLK_gater: BUFGCE 
	port map(
	I	=>clk, 
	CE	=>clock_enable, 
	O	=>i_clk
	);

	
TM_control: entity noc_v1_00_a.TM_control(behavioral)
	port map(
	clk					=> clk,
	reset					=> reset,
	CLK_GATE_FREQ		=> CLK_GATE_FREQ,
	readout_finished	=> readout_finished,
	start_testing		=> start_testing,
	clock_enable		=> clock_enable,
	readout				=> readout
	);
		
test_mesh: entity noc_v1_00_a.interface_mesh(behavioral)
	port map(
	ungated_clk			=> clk,
	clk					=> i_clk,
	reset					=> reset,
	start_testing		=> start_testing,
	grant_l_33			=> grant,
	x_in_l_33			=> x_in,
	request_l_33		=> request,
	mux_select			=> mux_select,
	readout				=> readout,
	readout_finished	=> readout_finished,
	BRAM_enable			=> BRAM_enable,
	BRAM_addr			=> BRAM_addr,
	x_out_l_33			=> x_out,
	RTS_l_33			=> RTS,
	CTS_l_33			=> CTS,
	mux_o_packetcnt		=> mux_o_packetcnt,
	mux_o_tstamp		=> mux_o_tstamp
	);

HW_PROC: process(clk,reset) is
	variable recv_state, nrecv_state	:RecvState;
	variable send_state, nsend_state	:SendState;
    
	begin       
        if reset = '0' then
			
			nrecv_state := IDLE;
			recv_state := IDLE;
			nsend_state := IDLE;
			send_state := IDLE;
			
			x_in <= (others=>'0');
			packet_in <= (others=>'0');
			packet_out <= (others=>'0');
			data_out <= (others => '0');
			
			request <= '0';
			grant <= '0';
			packet_received_33 <= '0';
			packet_sendt <= '0';
			prev_send_packet <= '0';
            
			send_counter <= 0;
			
			recv_counter <= -1;
		           
        elsif clk'event and clk='1' then  
		---------------------------
		-- Network Communication --
		---------------------------
		
		-------------------------------------
		-- Receiving a packet to router 33 --
		-------------------------------------
		
		case recv_state is
			when IDLE =>
			packet_received_33 <= '0';
			if RTS = '1' then
				nrecv_state := RECEIVE;
				grant <= '1';
			else
				nrecv_state := IDLE;
				grant <= '0';
			end if;
	
			when RECEIVE =>
			if recv_counter = 8 then
				nrecv_state := IDLE;
				grant <= '0';
				recv_counter <= -1;
				data_out <= packet_out;
				packet_received_33 <= '1';
			else
				if recv_counter = 6 or recv_counter = 7 then
					grant <= '0';
				end if;
				recv_counter <= recv_counter + 1;
				packet_out <= packet_out(55 downto 0) & x_out;
			end if;
		end case;
			
        --------------------------------------
        -- Sending a packet in on router 33 --
		--------------------------------------
		
		case send_state is
			when IDLE =>
			packet_sendt <= '0';
			if send_packet = '1' and prev_send_packet = '0' then
				packet_in <= data_in; --read package from data_in
				nsend_state := HOLD;
				request <= '1';
			end if;
				
			when HOLD => 
			packet_sendt <= '0';
			request <= '1';
			if CTS = '1' then
				nsend_state := SEND;
				send_counter <= send_counter + 1;
				packet_in <= packet_in(55 downto 0) & packet_in(63 downto 56);
			end if;
						
			when SEND =>			 
			if send_counter = 7 then
				if CTS = '1' then
					packet_in <= packet_in(55 downto 0) & packet_in(63 downto 56);
					nsend_state := IDLE;
					request <= '0';
					packet_sendt <= '1';
					send_counter <= 0;
				end if;
			else
				packet_in <= packet_in(55 downto 0) & packet_in(63 downto 56);
				send_counter <= send_counter + 1;
			end if;
					
			when others =>
			null;
		end case;					
		
		recv_state := nrecv_state;
		send_state := nsend_state;
		x_in <= packet_in(63 downto 56); 
		prev_send_packet <= send_packet;
		readout_SW <= readout;
		end if; 
end process;
end architecture;
