library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.proc_common_pkg.all;
entity user_logic is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    --USER generics added here
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_DWIDTH                       : integer              := 32;
    C_NUM_CE                       : integer              := 9
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
    --USER ports added here
    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Reset                   : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(0 to C_DWIDTH-1);
    Bus2IP_BE                      : in  std_logic_vector(0 to C_DWIDTH/8-1);
    Bus2IP_RdCE                    : in  std_logic_vector(0 to C_NUM_CE-1);
    Bus2IP_WrCE                    : in  std_logic_vector(0 to C_NUM_CE-1);
    IP2Bus_Data                    : out std_logic_vector(0 to C_DWIDTH-1);
    IP2Bus_Ack                     : out std_logic;
    IP2Bus_Retry                   : out std_logic;
    IP2Bus_Error                   : out std_logic;
    IP2Bus_ToutSup                 : out std_logic
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );

  attribute SIGIS : string;
  attribute SIGIS of Bus2IP_Clk    : signal is "CLK";
  attribute SIGIS of Bus2IP_Reset  : signal is "RST";

end entity user_logic;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of user_logic is

  --USER signal declarations added here, as needed for user logic
	signal clk,reset                    :std_logic;
	signal data_in                      :std_logic_vector(63 downto 0);
	signal send_packet                  :std_logic;
	signal mux_select							:std_logic_vector(3 downto 0);
	signal BRAM_enable						:std_logic;
	signal BRAM_addr							:std_logic_vector(8 downto 0);
	signal readout_finished					:std_logic;
	signal CLK_GATE_FREQ						:std_logic_vector(3 downto 0);
	signal start_testing						:std_logic;
	signal data_out							:std_logic_vector(63 downto 0);
	signal packet_received_33				:std_logic;
	signal packet_sendt						:std_logic;
	signal mux_o_packetcnt					:std_logic_vector(31 downto 0);
	signal mux_o_tstamp						:std_logic_vector(31 downto 0);
	signal readout_SW							:std_logic;
  ------------------------------------------
  -- Signals for user logic slave model s/w accessible register example
  ------------------------------------------
  signal slv_reg0                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg1                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg2                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg3                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg4                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg5                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg6                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg7                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg8                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg_write_select           : std_logic_vector(0 to 8);
  signal slv_reg_read_select            : std_logic_vector(0 to 8);
  signal slv_ip2bus_data                : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_read_ack                   : std_logic;
  signal slv_write_ack                  : std_logic;

begin
	--START OF CONFIG REGISTERS
	--Assigning SLV_REG_4
	clk 					<= Bus2IP_Clk;
	mux_select 			<= slv_reg4(28 to 31);
	send_packet 		<= slv_reg4(27);
	readout_finished 	<= slv_reg4(23);
	--BRAM_addr			<= slv_reg4(15 to 19);
	CLK_GATE_FREQ 		<= slv_reg4(8 to 11);
	start_testing 		<= slv_reg4(7);
	BRAM_enable			<= slv_reg4(7);
	reset 				<= slv_reg4(3);
	
	--Assigning SLV_REG_5
	BRAM_addr			<= slv_reg5(23 to 31);
	
	--END OF CONFIG REGISTERS

	data_in <= slv_reg0 & slv_reg1;
	
	interface : entity noc_v1_00_a.interface(behavioral)
		port map(
		clk 					=> clk,
		reset					=> reset,
		data_in				=> data_in,
		send_packet			=> send_packet,
		mux_select			=> mux_select,
		BRAM_enable			=> BRAM_enable,
		BRAM_addr			=> BRAM_addr,
		readout_finished	=> readout_finished,
		CLK_GATE_FREQ 		=> CLK_GATE_FREQ,
		start_testing		=> start_testing,
		data_out				=> data_out,
		packet_received_33=> packet_received_33,
		packet_sendt		=> packet_sendt,
		mux_o_packetcnt	=> mux_o_packetcnt,
		mux_o_tstamp		=> mux_o_tstamp,
		readout_SW			=> readout_SW
	);

  
  slv_reg_write_select <= Bus2IP_WrCE(0 to 8);
  slv_reg_read_select  <= Bus2IP_RdCE(0 to 8);
  slv_write_ack        <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1) or Bus2IP_WrCE(2) or Bus2IP_WrCE(3) or Bus2IP_WrCE(4) or Bus2IP_WrCE(5) or Bus2IP_WrCE(6) or Bus2IP_WrCE(7) or Bus2IP_WrCE(8);
  slv_read_ack         <= Bus2IP_RdCE(0) or Bus2IP_RdCE(1) or Bus2IP_RdCE(2) or Bus2IP_RdCE(3) or Bus2IP_RdCE(4) or Bus2IP_RdCE(5) or Bus2IP_RdCE(6) or Bus2IP_RdCE(7) or Bus2IP_RdCE(8);

  -- implement slave model register(s)
  SLAVE_REG_WRITE_PROC : process( Bus2IP_Clk ) is
  begin

    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
	 
		slv_reg7 <= mux_o_packetcnt;
		slv_reg8 <= mux_o_tstamp;

		if packet_received_33 = '1' then
			slv_reg6(27) <= '1';
			slv_reg2 <= data_out(63 downto 32);
			slv_reg3 <= data_out(31 downto 0);
		end if;

		if readout_SW = '1' then
			slv_reg6(31) <= '1';
		end if;
	 
      if Bus2IP_Reset = '1' then
        slv_reg0 <= (others => '0');
        slv_reg1 <= (others => '0');
        slv_reg2 <= (others => '0');
        slv_reg3 <= (others => '0');
        slv_reg4 <= (others => '0');
        slv_reg5 <= (others => '0');
        slv_reg6 <= (others => '0');
        slv_reg7 <= (others => '0');
        slv_reg8 <= (others => '0');
      else
        case slv_reg_write_select is
          when "100000000" =>
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg0(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "010000000" =>
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg1(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
--          when "001000000" =>
--            for byte_index in 0 to (C_DWIDTH/8)-1 loop
--              if ( Bus2IP_BE(byte_index) = '1' ) then
--                slv_reg2(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
--              end if;
--            end loop;
--          when "000100000" =>
--            for byte_index in 0 to (C_DWIDTH/8)-1 loop
--              if ( Bus2IP_BE(byte_index) = '1' ) then
--                slv_reg3(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
--              end if;
--            end loop;
          when "000010000" =>
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg4(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "000001000" =>
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg5(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "000000100" =>
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg6(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
--          when "000000010" =>
--            for byte_index in 0 to (C_DWIDTH/8)-1 loop
--              if ( Bus2IP_BE(byte_index) = '1' ) then
--                slv_reg7(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
--              end if;
--            end loop;
--          when "000000001" =>
--            for byte_index in 0 to (C_DWIDTH/8)-1 loop
--              if ( Bus2IP_BE(byte_index) = '1' ) then
--                slv_reg8(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
--              end if;
--            end loop;
          when others => null;
        end case;
      end if;
    end if;

  end process SLAVE_REG_WRITE_PROC;

  -- implement slave model register read mux
  SLAVE_REG_READ_PROC : process( slv_reg_read_select, slv_reg0, slv_reg1, slv_reg2, slv_reg3, slv_reg4, slv_reg5, slv_reg6, slv_reg7, slv_reg8 ) is
  begin

    case slv_reg_read_select is
      when "100000000" => slv_ip2bus_data <= slv_reg0;
      when "010000000" => slv_ip2bus_data <= slv_reg1;
      when "001000000" => slv_ip2bus_data <= slv_reg2;
      when "000100000" => slv_ip2bus_data <= slv_reg3;
      when "000010000" => slv_ip2bus_data <= slv_reg4;
      when "000001000" => slv_ip2bus_data <= slv_reg5;
      when "000000100" => slv_ip2bus_data <= slv_reg6;
      when "000000010" => slv_ip2bus_data <= slv_reg7;
      when "000000001" => slv_ip2bus_data <= slv_reg8;
      when others => slv_ip2bus_data <= (others => '0');
    end case;

  end process SLAVE_REG_READ_PROC;

  ------------------------------------------
  -- Example code to drive IP to Bus signals
  ------------------------------------------
  IP2Bus_Data        <= slv_ip2bus_data;

  IP2Bus_Ack         <= slv_write_ack or slv_read_ack;
  IP2Bus_Error       <= '0';
  IP2Bus_Retry       <= '0';
  IP2Bus_ToutSup     <= '0';

end IMP;
